package LogViewer;

import java.io.File;
import java.nio.file.Files;
import java.util.HashSet;
import java.util.stream.Stream;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.layout.BorderPane;
import javafx.stage.FileChooser;

public class LogviewerController {
	
	@FXML 
	BorderPane pane;
	@FXML
	TextField path;
	@FXML
	TableView<Entry> all;
	@FXML
	TableView<Entry> debug;
	@FXML
	TableView<Entry> info;
	@FXML
	TableView<Entry> warn;
	@FXML
	TableView<Entry> error;
	@FXML
	ComboBox<String> tagSelect;
	
	private final HashSet<String> tags = new HashSet<>();
	
	private ObservableList<Entry> all_data = FXCollections.observableArrayList();
	private ObservableList<Entry> debug_data = FXCollections.observableArrayList();
	private ObservableList<Entry> info_data = FXCollections.observableArrayList();
	private ObservableList<Entry> warn_data = FXCollections.observableArrayList();
	private ObservableList<Entry> error_data = FXCollections.observableArrayList();
	
	@FXML
    public void initialize() {
        all.getColumns().get(0).setCellValueFactory(new PropertyValueFactory<>("mode"));
        all.getColumns().get(1).setCellValueFactory(new PropertyValueFactory<>("time"));
        all.getColumns().get(2).setCellValueFactory(new PropertyValueFactory<>("tag"));
        all.getColumns().get(3).setCellValueFactory(new PropertyValueFactory<>("message"));
        debug.getColumns().get(0).setCellValueFactory(new PropertyValueFactory<>("time"));
        debug.getColumns().get(1).setCellValueFactory(new PropertyValueFactory<>("tag"));
        debug.getColumns().get(2).setCellValueFactory(new PropertyValueFactory<>("message"));
        info.getColumns().get(0).setCellValueFactory(new PropertyValueFactory<>("time"));
        info.getColumns().get(1).setCellValueFactory(new PropertyValueFactory<>("tag"));
        info.getColumns().get(2).setCellValueFactory(new PropertyValueFactory<>("message"));
        warn.getColumns().get(0).setCellValueFactory(new PropertyValueFactory<>("time"));
        warn.getColumns().get(1).setCellValueFactory(new PropertyValueFactory<>("tag"));
        warn.getColumns().get(2).setCellValueFactory(new PropertyValueFactory<>("message"));
        error.getColumns().get(0).setCellValueFactory(new PropertyValueFactory<>("time"));
        error.getColumns().get(1).setCellValueFactory(new PropertyValueFactory<>("tag"));
        error.getColumns().get(2).setCellValueFactory(new PropertyValueFactory<>("message"));
        tagSelect.getItems().add("alle");
        tagSelect.setValue("alle");
    }
	
	@FXML
	protected void selectFile() {
		tags.clear();
		tags.add("alle");
		final FileChooser fileChooser = new FileChooser();
		File file = fileChooser.showOpenDialog(pane.getScene().getWindow());
        if (file != null) {
            openFile(file);
        }
        tagSelect.getItems().clear();
        tags.forEach(tag -> tagSelect.getItems().add(tag));
        tagSelect.setValue("alle");
	}
	
	@FXML
	protected void onTagSelect() {
		final String tag = tagSelect.getSelectionModel().getSelectedItem();
		if(null == tag || tag.isEmpty())
			return;
		clearTables();
		if("alle".equals(tag)) {
			all.getItems().addAll(all_data);
			debug.getItems().addAll(debug_data);
			info.getItems().addAll(info_data);
			warn.getItems().addAll(warn_data);
			error.getItems().addAll(error_data);
		} else {
			all_data.stream().filter(entry -> entry.getTag().equals(tag)).forEach(entry -> all.getItems().add(entry));
			debug_data.stream().filter(entry -> entry.getTag().equals(tag)).forEach(entry -> debug.getItems().add(entry));
			info_data.stream().filter(entry -> entry.getTag().equals(tag)).forEach(entry -> info.getItems().add(entry));
			warn_data.stream().filter(entry -> entry.getTag().equals(tag)).forEach(entry -> warn.getItems().add(entry));
			error_data.stream().filter(entry -> entry.getTag().equals(tag)).forEach(entry -> error.getItems().add(entry));
		}
	}
	
	
	private void openFile(final File file) {
		path.setText(file.getAbsolutePath());
		clearData();
		clearTables();
		try {
			Files.lines(file.toPath())
				.map(Entry::fromLine)
				.peek(entry -> tags.add(entry.getTag()))
				.forEach(this::addEntry);
		} catch (Exception e) {
			e.printStackTrace();
		}
		all_data.addAll(all.getItems());
		debug_data.addAll(debug.getItems());
		info_data.addAll(info.getItems());
		warn_data.addAll(warn.getItems());
		error_data.addAll(error.getItems());
	}
	
	private void clearTables() {
		Stream.of(all,debug,info,warn,error)
			.map(TableView::getItems)
			.forEach(list -> list.clear());
	}
	
	private void clearData() {
		Stream.of(all_data,debug_data,info_data,warn_data,error_data).forEach(list -> list.clear());
	}
	
	private void addEntry(Entry entry) {
		all.getItems().add(entry);
		if(entry.hasMode("DEBUG"))
			debug.getItems().add(entry);
		if(entry.hasMode("INFO"))
			info.getItems().add(entry);
		if(entry.hasMode("WARN"))
			warn.getItems().add(entry);
		if(entry.hasMode("ERROR"))
			error.getItems().add(entry);
	}
}
